#!  /usr/bin/perl

#  This calculates bodyfat based on multiple measurement points.
#  Change the following variable to say what measurement system
#  you want to use -- lbs, kgs, etc.  Doesn't matter what it 
#  is, it's just used to print out messages, not for calculations.

$weightsarein = "lbs";                                                                  

sub calculate_male 
{
   my $chest, $abdomen, $thigh, $age, $X3, $X4;
   my $density, $percent;
   
   ($chest, $abdomen, $thigh, $age) = @_;
   $X3 = $chest + $abdomen + $thigh;
   $X4 = $age;
   $density = 1.1093800 - (0.0008267 * $X3) + (0.0000016 * $X3**2) - (0.0002574 * $X4);
   $percent =  (495 / $density) - 450;
   return $percent;
}

sub calculate_female 
{
   my $triceps, $supra, $thigh, $age, $X3, $X4;
   my $density, $percent;

   ($triceps, $supra, $thigh, $age) = @_;
   $X3 = $triceps + $supra + $thigh;
   $X4 = $age;
   $density = 1.099421 - (0.0009929* $X3) + (0.0000023 * $X3**2) - (0.0001392 * $X4);
   $percent = (457 / $density) - 414.2;
   return $percent;
}

sub ask_sex 
{
   print "Are you male or female (m/f)? ";
   chomp($sex = readline(STDIN));
   return $sex;
}

sub ask_measurement 
{
   $measure = shift();
   print "What is your $measure skinfold measurement (in mm)? ";
   chomp($dim = readline(STDIN));
   return $dim;  
}

sub ask_age
{
   print "What is your age (in years)? ";
   chomp($age = readline(STDIN));
   return $age;  
}

sub ask_weight
{
   print "What is your bodyweight ($weightsarein)? ";
   chomp($weight = readline(STDIN));
   return $weight;  
}

sub ask_target
{
   print "What is your target bodyfat percentage (ie 10.0)? ";
   chomp($pct = readline(STDIN));
   return $pct;  
}

sub ask_gender_specific 
{
   my $gender, $age;
   ($gender, $age) = @_;

   if($gender =~ /^[mM].*/)
   {
      $chest = ask_measurement("chest");
      $abdomen = ask_measurement("abdomen");
      $thigh = ask_measurement("thigh");
      $percent = calculate_male($chest, $abdomen, $thigh, $age);
   }
   else
   {
      $triceps = ask_measurement("triceps");
      $supra = ask_measurement("suprailiac");
      $thigh = ask_measurement("thigh");
      $percent = calculate_female($triceps, $supra, $thigh, $age);
   }
   return $percent;
}

$age = ask_age();
$sex = ask_sex();
$weight = ask_weight();
$targetpct = ask_target();
$percent = ask_gender_specific($sex, $age);
$tbf = ($percent / 100) * $weight;
$lbm = $weight - $tbf;
$targetbf = ($targetpct / 100) * $weight;

print "\n";
printf("Your current bodyweight is: %5.2f %s\n", $weight, $weightsarein);
printf("Your bodyfat percentage is: %5.2f%%\n", $percent);
printf("Your lean body mass is    : %5.2f %s\n", $lbm, $weightsarein);
printf("Your total bodyfat is     : %5.2f %s\n", $tbf, $weightsarein);
if($targetbf < $tbf)
{
   printf("You must lose %0.2f %s to reach your target of %0.2f%% bodyfat\n",
         $tbf - $targetbf, $weightsarein, $targetpct);
}

